;;
;; qr-payment.scm
;;
;; QR payment generator.
;;
;; ISC License
;;
;; Copyright 2023-2025 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit qr-payment))

(module
    qr-payment
    (
     make-qrp
     make-brmlab-qrp
     make-brmlab-qrp-svg-string
     )

  (import scheme
	  (chicken format)
	  (chicken string)
	  (chicken base)
	  util-io)

  (define (make-empty-qrp . vs)
    (let ((v (if (null? vs) "1.0" (car vs))))
      (list v "SPD")))

  (define (add-field-to-qrp qrp key value)
    (cons (format "~A:~A" key value)
	  qrp))

  (define (serialize-qrp qrp)
    (string-intersperse (reverse qrp) "*"))

  (define (ensure-amount-format amt)
    (let* ((n (if (string? amt)
		  (string->number amt)
		  amt))
	   (s (number->string n))
	   (f (string-split s "."))
	   (i? (null? (cdr f))))
      (format "~A.~A"
	      (car f)
	      (if i?
		  "00"
		  (substring
		   (string-append (cadr f) "0")
		   0 2)))))

  (define (make-qrp iban amt cc vs msg)
    (let loop ((keys '(ACC AM CC MSG X-VS))
	       (vals (list iban (ensure-amount-format amt) cc msg vs))
	       (qrp (make-empty-qrp)))
      (if (null? keys)
	  (serialize-qrp qrp)
	  (loop (cdr keys)
		(cdr vals)
		(add-field-to-qrp qrp (car keys) (car vals))))))

  (define (make-brmlab-qrp amt cc vs)
    (let ((iban (if (equal? cc "CZK")
		    "CZ0520100000002500079551"
		    (if (equal? cc "EUR")
			"CZ9320100000002100079552"
			(error "Invalid currency!")))))
      (make-qrp iban amt cc vs "Brmlab")))

  (define (qrp-create-svg-string qrps)
    (let-values
	(((ec ol)
	  (get-process-exit+output-lines
	   "qrencode"
	   "-t" "svg"
	   "--inline"
	   "-o" "-"
	   "-l" "M"
	   qrps)))
      (if (eq? ec 0)
	  (string-intersperse ol "\n")
	  #f)))

  (define (make-brmlab-qrp-svg-string amt cc vs)
    (qrp-create-svg-string
     (make-brmlab-qrp amt cc vs)))

  )
